﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FixOneDriveDuplicates
{
    public partial class Form1 : Form
    {
        string backupPath;
        string checkPath;
        Keep keep;
        enum Keep { newest, oldest }

        private delegate void updateEvent(string text);
        private event updateEvent _updateTextbox;
        
        public Form1()
        {
            _updateTextbox += Form1__updateTextbox;
            InitializeComponent();
            label1.Text = "This software will remove duplicated files with the name " + Environment.MachineName + " in the end\r\nYou will decide if you want to keep the newest or the oldest";
        }

        private void Form1__updateTextbox(string text)
        {
            textBox1.AppendText(text+"\r\n");
        }
        

        private void buttonFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if(fbd.ShowDialog() == DialogResult.OK)
            {
                if(Directory.Exists(fbd.SelectedPath))
                {
                    
                    DialogResult dr = MessageBox.Show("Do you want to keep the newest file or the oldest file? Click Yes for Newest, No for oldest, and Cancel for Cancel", "Question", MessageBoxButtons.YesNoCancel);
                    if (dr == DialogResult.Cancel) return;
                    if (dr == DialogResult.Yes) keep = Keep.newest; else keep = Keep.oldest;
                    DialogResult dr2 = MessageBox.Show("Do you want to keep a backup of the deleted file?", "Important", MessageBoxButtons.YesNo);
                    if (dr2 == DialogResult.Yes)
                    {
                        FolderBrowserDialog fbd2 = new FolderBrowserDialog();
                        if (fbd2.ShowDialog() == DialogResult.OK)
                        { backupPath = fbd2.SelectedPath; }
                    }
                    checkPath = fbd.SelectedPath;
                    Thread t = new Thread(new ThreadStart(process));
                    t.IsBackground = true;
                    t.Start();
                    buttonFolder.Hide();
                    label1.Hide();
                    textBox1.Show();
                }
            }
        }

        private void process()
        {
            DirectoryInfo dir = new DirectoryInfo(checkPath);

            FileInfo[] files = dir.GetFiles("*-" + Environment.MachineName + "*", SearchOption.AllDirectories);
            Log("Found " + files.Length + " files matching *-" + Environment.MachineName + "* ");
            foreach(FileInfo file in files)
            {
                try
                {
                    string orig = StringExtensions.ReplaceInsensitive(file.Name, "-" + Environment.MachineName, "");
                    FileInfo f = new FileInfo(file.DirectoryName + "\\" + orig);
                    Log(f.FullName + " KEEPING " + keep);
                    if (keep == Keep.newest)
                    {
                        Log("ORIG: " + f.LastWriteTime + ", FILE: " + file.LastWriteTime);
                        if (f.LastWriteTime < file.LastWriteTime)
                        {
                            Log(keep + " file is: " + file.Name);
                            // we keep the file with pc name in it
                            string newname = f.Name;
                            if (backupPath != "")
                            {
                                DirectoryInfo d = Directory.CreateDirectory(backupPath + "\\" + file.DirectoryName.Substring(checkPath.Length));
                                f.CopyTo(d.FullName + "\\" + f.Name,true);
                                Log("Backed up " + d.FullName + "\\" + f.Name);
                            }
                            f.Delete();
                            file.MoveTo(file.DirectoryName + "\\" + newname);
                        }
                        else
                        {
                            Log(keep + " f is: " + f.Name);
                            // we keep the file without pc name in it
                            if (backupPath != "")
                            {
                                DirectoryInfo d = Directory.CreateDirectory(backupPath + "\\" + file.DirectoryName.Substring(checkPath.Length));
                                file.CopyTo(d.FullName + "\\" + file.Name, true);
                                Log("Backed up " + d.FullName + "\\" + file.Name);
                            }
                             file.Delete();
                        }
                    }
                    else
                    {
                        Log("ORIG: " + f.LastWriteTime + ", FILE: " + file.LastWriteTime);
                        if (f.LastWriteTime < file.LastWriteTime)
                        {
                            Log(keep + " f is: " + f.Name);
                            // we keep the file without pc name in it
                            if (backupPath != "")
                            {
                                DirectoryInfo d = Directory.CreateDirectory(backupPath + "\\" + file.DirectoryName.Substring(checkPath.Length));
                                file.CopyTo(d.FullName + "\\" + file.Name, true);
                                Log("Backed up " + d.FullName + "\\" + file.Name);
                            }
                            file.Delete();
                        }
                        else
                        {
                            Log(keep + " file is: " + file.Name);
                            // we keep the file with pc name in it
                            string newname = f.Name;
                            if (backupPath != "")
                            {
                                DirectoryInfo d = Directory.CreateDirectory(backupPath + "\\" + file.DirectoryName.Substring(checkPath.Length));
                                f.CopyTo(d.FullName + "\\" + f.Name, true);
                                Log("Backed up " + d.FullName + "\\" + f.Name);
                            }
                            f.Delete();
                            file.MoveTo(file.DirectoryName + "\\" + newname);
                        }
                    }
                }
                catch(Exception err)
                {
                    Log("Sorry, made a poh poh during check of file " + file.FullName + " [" + err.Message + "]");
                }
            }
        }

        private void Log(string text)
        {
            this.Invoke(_updateTextbox, new object[] { text });
        }
    }

    static public class StringExtensions
    {
        static public string ReplaceInsensitive(this string str, string from, string to)
        {
            str = Regex.Replace(str, from, to, RegexOptions.IgnoreCase);
            return str;
        }
    }
}
